#include "stdafx.h"
#include ".\rootwindow.h"

CRootWindow::CRootWindow(void)
{
	ZeroMemory(&m_bmDB, sizeof(m_bmDB));
	m_hWndBase = NULL;
	m_bTracking = FALSE;
	m_hBitmapDB = NULL;
	m_pbmpBits = NULL;
	m_closeAction = NULL;
	m_minimizeAction = NULL;
	m_maximizeAction = NULL;
	m_dataContext = NULL;
	m_RoundRegion = 0;	
	m_bTaskbar = FALSE;
	m_bCenterWindow = FALSE;
	m_alpha = 255;
}

CRootWindow::~CRootWindow(void)
{
	if (m_hBitmapDB)
	{
		::DeleteObject(m_hBitmapDB);
		m_hBitmapDB = NULL;
	}
}
void CRootWindow::SetRoundRegion(int round)
{
	m_RoundRegion = round;	
}
void CRootWindow::SetTaskbar(BOOL bTaskbar)
{
	m_bTaskbar = bTaskbar;
	if (IsWindow(m_hWndBase))
	{
		CWindow window = m_hWndBase;
		if (m_bTaskbar == TRUE)
		{
			window.ModifyStyleEx(0, WS_EX_TOOLWINDOW);
		}
		else
		{
			window.ModifyStyleEx(WS_EX_TOOLWINDOW, 0);
		}
	}
}
void CRootWindow::SetCenterWindow(BOOL bCenterWindow)
{
	m_bCenterWindow = bCenterWindow;
	if (IsWindow(m_hWndBase))
	{
		if (IsCenterWindow())
		{
			CWindow window = m_hWndBase;
			window.CenterWindow();
		}
	}
}
void CRootWindow::SetDataContext(IDataContext* datacontext)
{
	m_dataContext = datacontext;
}
void CRootWindow::Draw(HDC hDC)
{
	CWindow windowBase = m_hWndBase;
	if (windowBase.GetExStyle() & WS_EX_LAYERED)
	{
		_DrawLayer(hDC);
		return ;
	}

	CRect rcClient;
	GetClientRect(m_hWndBase, &rcClient);

	_UpdateBufferImage(hDC, rcClient.Width(), rcClient.Height());

	if (m_hBitmapDB)
	{
		HDC hMemDC = CreateCompatibleDC(hDC);
		HGDIOBJ old = SelectObject(hMemDC, m_hBitmapDB);

		FillSolidRect(hMemDC, 0, 0, rcClient.Width(), rcClient.Height(), RGB(0xff, 0xff, 0xff));
		m_rootContainer.OnDraw(hMemDC);		

		BitBlt(hDC, 0, 0, rcClient.Width(), rcClient.Height(), hMemDC, 0, 0, SRCCOPY);

		SelectObject(hMemDC, old);
		DeleteDC(hMemDC);
	}
	else
	{
		FillSolidRect(hDC, 0, 0, rcClient.Width(), rcClient.Height(), RGB(0xff, 0xff, 0xff));
		m_rootContainer.OnDraw(hDC);			
	}
}

void CRootWindow::_DrawLayer(HDC hDC)
{
	CWindow windowBase = m_hWndBase;

	if ((windowBase.GetExStyle() & WS_EX_LAYERED) == FALSE)
	{
		return;
	}

	CRect rcWindow;
	windowBase.GetWindowRect(&rcWindow);

	BOOL bGetDC = FALSE;
	if (hDC == NULL)
	{
		hDC = ::GetDC(NULL);
		bGetDC = TRUE;
	}
	_UpdateBufferImage(hDC, rcWindow.Width(), rcWindow.Height());

	HDC hMemory = ::CreateCompatibleDC(hDC);

	HBITMAP hbmpOld = (HBITMAP)::SelectObject(hMemory, m_hBitmapDB);	
	m_rootContainer.OnDraw(hMemory);
	//PreMultiplyRGBChannels(rcWindow.Width(), rcWindow.Height(), m_pbmpBits);

	SIZE szBmp = { rcWindow.Width(), rcWindow.Height() };
	POINT ptDest = { rcWindow.left, rcWindow.top }, ptSrc = { 0, 0 };
	BLENDFUNCTION bf = { AC_SRC_OVER, 0, (BYTE)(m_alpha * 100 / 100), AC_SRC_ALPHA };
	BOOL bUpdateWindow = UpdateLayeredWindow(m_hWndBase, hDC, &ptDest, &szBmp, hMemory, &ptSrc, 0, &bf, ULW_ALPHA);//ULW_OPAQUE//ULW_ALPHA

	::SelectObject(hMemory, hbmpOld);	
	DeleteDC(hMemory);
	if (bGetDC)
	{
		::ReleaseDC(NULL, hDC);
	}
}
BOOL CRootWindow::_UpdateBufferImage(HDC hDC, int cx, int cy)
{
	if (m_hBitmapDB)
	{
		BITMAP bm;
		GetObject(m_hBitmapDB, sizeof(bm), &bm);
		if (bm.bmWidth == cx && bm.bmHeight == cy)
		{
			DWORD dwLen = cx * cy * GetDoubleBufferBitCount()/8;
			ZeroMemory(m_pbmpBits, dwLen);
			return TRUE;
		}
		DeleteObject(m_hBitmapDB);
		m_hBitmapDB = NULL;
		m_pbmpBits = NULL;
	}
	BOOL bGetDC = FALSE;
	if (hDC == NULL)
	{
		hDC = ::GetDC(NULL);
		bGetDC = TRUE;
	}
	if (hDC)
	{
		BITMAPINFO bmi = { 0 };
		bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		bmi.bmiHeader.biPlanes = 1;
		bmi.bmiHeader.biBitCount = GetDoubleBufferBitCount();
		bmi.bmiHeader.biCompression = BI_RGB;
		bmi.bmiHeader.biWidth = cx;
		bmi.bmiHeader.biHeight = cy;

		m_hBitmapDB = ::CreateDIBSection(hDC, (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, (void **)&m_pbmpBits, NULL, 0);
		GetObject(m_hBitmapDB, sizeof(m_bmDB), &m_bmDB);

		if (bGetDC)
		{
			::ReleaseDC(NULL, hDC);
		}
	}	

	return (m_hBitmapDB != NULL);
}

IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnPaint)
{
	PAINTSTRUCT ps;
	HDC hDC = BeginPaint(m_hWndBase, &ps);
	if (hDC)
	{
		Draw(hDC);
	}
	EndPaint(m_hWndBase, &ps);
	return 0;
}
IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnEraseBkgnd)
{
	return TRUE;
}
IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnSize)
{
	int cx = LOWORD(lParam);
	int cy = HIWORD(lParam);

	m_rootContainer.MoveWindow(0, 0, cx, cy, FALSE);
	if (m_RoundRegion > 0)
	{		
		CRgn rgn_window;
		rgn_window.CreateRoundRectRgn(0, 0, cx+m_RoundRegion/10+1, cy+m_RoundRegion/10+1, m_RoundRegion, m_RoundRegion);
		
		::SetWindowRgn(m_hWndBase, rgn_window, TRUE);
	}
	::PostMessage(m_hWndBase, UM_REFRESH, 0, 0);
	return 0;
}
IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnMouseMove)
{ 
	if(m_bTracking != TRUE)	
	{
		TRACKMOUSEEVENT TrackEvent;
		TrackEvent.cbSize      = sizeof( TRACKMOUSEEVENT );
		TrackEvent.dwFlags     = TME_LEAVE;
		TrackEvent.hwndTrack   = m_hWndBase;
		TrackEvent.dwHoverTime = 1;
		m_bTracking = TrackMouseEvent( &TrackEvent );
	}
	bHandled = FALSE;
	return 0;
}
IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnMouseLeave)
{
	if (m_bTracking)
	{
		m_rootContainer.MouseAway();
		m_bTracking = FALSE;		
	}
	bHandled = FALSE;
	return 0;
}
IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnKillFocus)
{
	m_rootContainer.SetFocus(NULL);
	bHandled = FALSE;
	return 0;
}

IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnSyscommand) 
{
	if(SC_CLOSE == wParam)
	{
		if (m_closeAction != NULL)
		{
			BOOL bClose = TRUE;
			m_closeAction->OnWindowClosing(&bClose);
			if (bClose == TRUE)
			{
				DestroyWindow(m_hWndBase);
				return 0;
			}
			return 1;
		}
	}
	else if(SC_MINIMIZE == wParam)
	{
		if (m_minimizeAction != NULL)
		{
			m_minimizeAction->OnActionPerformed(NULL);
			return 0;
		}
	}
	if(SC_MAXIMIZE == wParam)
	{
		if (m_maximizeAction != NULL)
		{
			m_maximizeAction->OnActionPerformed(NULL);
			return 0;
		}
	}

	bHandled = FALSE;
	return 0;
}
IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnDestroy)
{
	this->OnDestroy();
	return 1;
}
IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnPreTranslateMessage)
{
	MSG* pMsg = (MSG*)wParam;
	return m_rootContainer.PreTranslateMessage(pMsg);
}

IMPLEMENT_MESSAGE_HANDLER(CRootWindow, OnUIRefresh)
{
	_DrawLayer(NULL);
	return 0;
}